==> thunderbird-91.5.1.source.tar.xz: thunderbird-91.5.1/toolkit/crashreporter/test/unit/test_crash_AsyncShutdown.js <==
/* Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/ */

// Test that AsyncShutdown report errors correctly

// Note: these functions are evaluated in their own process, hence the need
// to import modules into each function.

function setup_crash() {
  /* global AsyncShutdown */
  const { AsyncShutdown } = ChromeUtils.import(
    "resource://gre/modules/AsyncShutdown.jsm"
  );
  const { Services } = ChromeUtils.import(
    "resource://gre/modules/Services.jsm"
  );
  const { Promise } = ChromeUtils.import("resource://gre/modules/Promise.jsm");

  Services.prefs.setBoolPref("toolkit.asyncshutdown.testing", true);
  Services.prefs.setIntPref("toolkit.asyncshutdown.crash_timeout", 10);

  let TOPIC = "testing-async-shutdown-crash";
  let phase = AsyncShutdown._getPhase(TOPIC);
  phase.addBlocker("A blocker that is never satisfied", function() {
    dump("Installing blocker\n");
    let deferred = Promise.defer();
    return deferred.promise;
  });

  Services.obs.notifyObservers(null, TOPIC);
  dump(new Error().stack + "\n");
  dump("Waiting for crash\n");
}

function after_crash(mdump, extra) {
  info("after crash: " + extra.AsyncShutdownTimeout);
  let data = JSON.parse(extra.AsyncShutdownTimeout);
  Assert.equal(data.phase, "testing-async-shutdown-crash");
  Assert.equal(data.conditions[0].name, "A blocker that is never satisfied");
  // This test spawns subprocesses by using argument "-e" of xpcshell, so
  // this is the filename known to xpcshell.
  Assert.equal(data.conditions[0].filename, "-e");
}

// Test that AsyncShutdown + OS.File reports errors correctly, in a case in which
// the latest operation succeeded

function setup_osfile_crash_noerror() {
  const { Services } = ChromeUtils.import(
    "resource://gre/modules/Services.jsm"
  );
  const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
  const { Promise } = ChromeUtils.import("resource://gre/modules/Promise.jsm");

  Services.prefs.setIntPref("toolkit.asyncshutdown.crash_timeout", 1);
  Services.prefs.setBoolPref("toolkit.osfile.native", false);

  OS.File.profileBeforeChange.addBlocker(
    "Adding a blocker that will never be resolved",
    () => Promise.defer().promise
  );
  OS.File.getCurrentDirectory();

  Services.startup.advanceShutdownPhase(
    Services.startup.SHUTDOWN_PHASE_APPSHUTDOWN
  );
  dump("Waiting for crash\n");
}

function after_osfile_crash_noerror(mdump, extra) {
  info("after OS.File crash: " + extra.AsyncShutdownTimeout);
  let data = JSON.parse(extra.AsyncShutdownTimeout);
  let state = data.conditions[0].state;
  info("Keys: " + Object.keys(state).join(", "));
  Assert.equal(data.phase, "profile-before-change");
  Assert.ok(state.launched);
  Assert.ok(!state.shutdown);
  Assert.ok(state.worker);
  Assert.ok(!!state.latestSent);
  Assert.equal(state.latestSent[1], "getCurrentDirectory");
}

// Test that AsyncShutdown + OS.File reports errors correctly, in a case in which
// the latest operation failed

function setup_osfile_crash_exn() {
  const { Services } = ChromeUtils.import(
    "resource://gre/modules/Services.jsm"
  );
  const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
  const { Promise } = ChromeUtils.import("resource://gre/modules/Promise.jsm");

  Services.prefs.setIntPref("toolkit.asyncshutdown.crash_timeout", 1);
  Services.prefs.setBoolPref("toolkit.osfile.native", false);

  OS.File.profileBeforeChange.addBlocker(
    "Adding a blocker that will never be resolved",
    () => Promise.defer().promise
  );
  OS.File.read("I do not exist");

  Services.startup.advanceShutdownPhase(
    Services.startup.SHUTDOWN_PHASE_APPSHUTDOWN
  );
  dump("Waiting for crash\n");
}

function after_osfile_crash_exn(mdump, extra) {
  info("after OS.File crash: " + extra.AsyncShutdownTimeout);
  let data = JSON.parse(extra.AsyncShutdownTimeout);
  let state = data.conditions[0].state;
  info("Keys: " + Object.keys(state).join(", "));
  Assert.equal(data.phase, "profile-before-change");
  Assert.ok(!state.shutdown);
  Assert.ok(state.worker);
  Assert.ok(!!state.latestSent);
  Assert.equal(state.latestSent[1], "read");
}

add_task(async function run_test() {
  await do_crash(setup_crash, after_crash);
  await do_crash(setup_osfile_crash_noerror, after_osfile_crash_noerror);
  await do_crash(setup_osfile_crash_exn, after_osfile_crash_exn);
});

==> thunderbird-115.2.0.source.tar.xz: thunderbird-115.2.0/toolkit/crashreporter/test/unit/test_crash_AsyncShutdown.js <==
/* Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/ */

// Test that AsyncShutdown report errors correctly

// Note: these functions are evaluated in their own process, hence the need
// to import modules into each function.

function setup_crash() {
  const { AsyncShutdown } = ChromeUtils.importESModule(
    "resource://gre/modules/AsyncShutdown.sys.mjs"
  );
  const { PromiseUtils } = ChromeUtils.importESModule(
    "resource://gre/modules/PromiseUtils.sys.mjs"
  );

  Services.prefs.setBoolPref("toolkit.asyncshutdown.testing", true);
  Services.prefs.setIntPref("toolkit.asyncshutdown.crash_timeout", 10);

  let TOPIC = "testing-async-shutdown-crash";
  let phase = AsyncShutdown._getPhase(TOPIC);
  phase.addBlocker("A blocker that is never satisfied", function () {
    dump("Installing blocker\n");
    let deferred = PromiseUtils.defer();
    return deferred.promise;
  });

  Services.obs.notifyObservers(null, TOPIC);
  dump(new Error().stack + "\n");
  dump("Waiting for crash\n");
}

function after_crash(mdump, extra) {
  info("after crash: " + extra.AsyncShutdownTimeout);
  let data = JSON.parse(extra.AsyncShutdownTimeout);
  Assert.equal(data.phase, "testing-async-shutdown-crash");
  Assert.equal(data.conditions[0].name, "A blocker that is never satisfied");
  // This test spawns subprocesses by using argument "-e" of xpcshell, so
  // this is the filename known to xpcshell.
  Assert.equal(data.conditions[0].filename, "-e");
}

// Test that AsyncShutdown + IOUtils reports errors correctly.,

function setup_ioutils_crash() {
  const { PromiseUtils } = ChromeUtils.importESModule(
    "resource://gre/modules/PromiseUtils.sys.mjs"
  );

  Services.prefs.setIntPref("toolkit.asyncshutdown.crash_timeout", 1);

  IOUtils.profileBeforeChange.addBlocker(
    "Adding a blocker that will never be resolved",
    () => PromiseUtils.defer().promise
  );

  Services.startup.advanceShutdownPhase(
    Services.startup.SHUTDOWN_PHASE_APPSHUTDOWN
  );
  dump("Waiting for crash\n");
}

function after_ioutils_crash(mdump, extra) {
  info("after IOUtils crash: " + extra.AsyncShutdownTimeout);
  let data = JSON.parse(extra.AsyncShutdownTimeout);
  Assert.equal(
    data.phase,
    "IOUtils: waiting for profileBeforeChange IO to complete"
  );
}

add_task(async function run_test() {
  await do_crash(setup_crash, after_crash);
  await do_crash(setup_ioutils_crash, after_ioutils_crash);
});
